var App = (function() {
    var me = {};
    var db = window.openDatabase(Config.pkg, "1.0", Config.pkg + ".db", Config.dbSize);
    var autoloaded = false;
    var scrollLock = false;
    me.load = function() {
        me.scroll = new iScroll('content', {
          hScroll:false, hScrollbar:false, vScrollbar:false,
          snap:false,
          onScrollMove: function() {
                scrollLock = true;
          },
          onScrollEnd: function() {
                scrollLock = false;
          }
        });
        $('#refresh').bind('touchstart touchend mouseup', touchHandler(function(e) {
            db.transaction(loadNewItems, function() {
                showLoading(false);
                alert('Error loading Items');
            });
        }));
        try {
            db.transaction(setup);
            db.transaction(cleanOld);
            db.transaction(loadCache);
        } catch(e) {
            showLoading(false);
            alert('An error occured, please restart the application.');
        }
    };
    function setup(tx) {
        tx.executeSql('CREATE TABLE IF NOT EXISTS Content (id UNIQUE, title, image, author, source, link, feed, date, read DEFAULT 0, deleted DEFAULT 0)');
    }
    function loadCache(tx) {
        tx.executeSql('SELECT * FROM Content WHERE deleted=0 ORDER BY date DESC', [], loadCacheSuccess, loadCacheFailure);
    }
    function loadCacheSuccess(tx, data) {
        $("#content ul li").remove();
        showLoading(true);
        if  (autoloaded === false) {
            loadNewItems();
            autoloaded = true;
        }
        var items = data.rows;
        for (var i = 0; i < items.length; i++) {
            var item = items.item(i);
            var li = $("<li class='" + ( item.read === 1 ? "read" : "unread" ) + "' data-link='" + item.link + "'></li>" ); 
            li.append($("<div class='headline'><h1>" + item.title + "</h1><h2>" + ( item.source ? item.source : "" ) + "</h2></div></li>"));
            var d = item.date.replace('T', ' ').replace('-', '/');
            d = humanized_time_span(new Date(d + ' UTC'));
            if (d === 'NaN years ago') {
                d = '1 hour ago';
            }
            li.append($("<span>" + d + "</span>"));
            if (item.image) {
                li.append($("<div class='image'><img src='" + item.image + "'></div>"));
            }
            var right = $("<div class='right'><div class='corner'></div></div>");
            right.bind('touchstart touchend scroll mouseup', touchHandler(function(e) {
                var p = getParent(getParent($(e.target)));
                var url = p.attr('data-link');
                db.transaction(function(tx) {
                    if (p.hasClass("read")) {
                        remove(tx, url);
                    } else {
                        markRead(tx, url, false);
                    }
                });
            }));
            var left = $("<div class='left'></div>");
            left.bind('touchstart touchend scroll mouseup', touchHandler(function(e) {
                var url = getParent($(e.target)).attr('data-link');
                db.transaction(function(tx) {
                    markRead(tx, url, true);
                });
                return false;
            }));
            li.append(right);
            li.append(left);
            $("#content ul").append(li);
        }
        showLoading(false);
    }
    function loadCacheFailure() {
        alert('Error loading items.');
        showLoading(false);
    }
    function loadNewItems() {
        showLoading(true);
        db.transaction(function(tx) {
            var last;
            tx.executeSql('SELECT MAX(id) as last FROM Content', [], function(tx, res) {
                try {
                    last = res.rows.item(0).last;
                } catch(e) {alert(e.getMessage());}
                $.getJSON('http://thinkap.net/feed/' + Config.pkg + ( last ? '?id=' + last : '?' ) + '&callback=?', function(items) {
                    var sql = 'INSERT INTO Content (id, title, image, author, source, link, feed, date, read) VALUES (?, ?, ?, ?, ?, ?, ?, ?, 0)';
                    var vals = [];
                    _.each(items, function(item) {
                        db.transaction(function(tx2) {
                            tx2.executeSql(sql, [item.id, item.title, item.image, item.author, item.source, item.link, item.feed, item.date]);
                        });
                    });
                    setTimeout(function() {
                        db.transaction(loadCache);
                    }, 100);
                });
            }, function() {
                alert('Error getting lastest ID.');
            });
        });
    }
    function touchHandler(f) {
        return function(e) {
            if (e.type === 'touchend' || e.type === 'mouseup') {
                if (!scrollLock) {
                    f(e);
                }
            }
        };
    }
    function showLoading(on) {
        if (on) {
            if ($("canvas").length === 0) {
                me.sonic = new Sonic({
                    width: 240,
                    height: 240,
                    stepsPerFrame: 1,
                    trailLength: 1,
                    pointDistance: 0.02,
                    fps: 30,
                    fillColor: '#05E2FF',
                    step: function(point, index) {
                        this._.beginPath();
                        this._.moveTo(point.x, point.y);
                        this._.arc(point.x, point.y, index * 20, 0, Math.PI*2, false);
                        this._.closePath();
                        this._.fill();
                    },
                    path: [
                        ['arc', 120, 120, 100, 0, 360]
                    ]
                });
                me.sonic.play();
                $(document.body).append(me.sonic.canvas);
            }
        } else {
            setTimeout(function() {
                $("canvas").remove();
                me.scroll.refresh();
            }, 800);
        }
    }
    function markRead(tx, url, doOpen) {
        var open = doOpen || false;
        function d() {
            $("li[data-link='" + url + "']").addClass("read");
            console.log(open);
            try {
                if (open) {
                    window.Opener.open(url);
                }
            } catch(e) {
                if (open) {
                    window.open(url, '_top');
                }
            }
        }
        tx.executeSql("UPDATE Content SET read=1 WHERE link=?", [url], d, d);
    }
    function remove(tx, url) {
        tx.executeSql("UPDATE Content SET deleted=1 WHERE link=?", [url]);
        $("li[data-link='"+url+"']").remove();
        me.scroll.refresh();
    }
    function getParent(d) {
        return $($(d)[0].parentNode);
    }
    function cleanOld(tx) {
        tx.executeSql("DELETE FROM Content WHERE id not in (select id from content order by date desc limit 50)");
    }
    me._SSNavigate = function() {
        var first = $($('li')[0]).attr('data-link');
        db.transaction(function(tx) {
            markRead(tx, first, true);
        });
    };
    me._SSClear = function() {
        db.transaction(function(tx) {
            tx.executeSql('delete from content where 1=1');
            autoloaded = false;
            db.transaction(loadCache);
        });
    };
    me._SSScroll = function() {
        me.scroll.scrollTo(0,400,0,true);
    };
    return me;
})();
